/////////////////////////////////////////////////////////////////////////////
// This software is distributed under the terms of the General Public License.
//
// Program : BuildIT!
// Authors : Artur Prusakiewicz & Michal Switala
// E-mail  : prusak@mat.uni.torun.pl
//           scisly@mat.uni.torun.pl
// WWW     : http://www.mat.uni.torun.pl/~scisly/buildit/
//           http://www.mat.uni.torun.pl/~prusak/buildit/
//////////////////////////////////////////////////////////////////////////////


#include "Tree.h"

#include <qprogdlg.h> 
#include <qstring.h>
#include <qfile.h>
#include <qtstream.h>
#include <qregexp.h>
#include <qdatetm.h>
#include <qmessagebox.h>

/*!
  \class Generator
  \brief This class is used to generate a source file for selected project

  This class generates source file (or files for applet) and write it to directory ~/.build/out <br>
  Files are: <br>
       \e your_project_name.java <br>
     and for applet there is additional file named: <br>
       \e your_project_name.html <br>
     which you can add to your html files to use created with BuildIT! applet
*/

class Generator
{
 public:
  /*!
    Constructor for class Generator, where parameters are: <br>
    val - number of all children in MyTree structure of project <br>
    widget - pointer to parent widget <br>
    home - absolute path to user directory with <b>BuildIT! </b> files (directorys: out, saves, tmp) <br>
    sys - absolute path to <b>BuildIT! </b> directory with Templates and documentation files <br>
    st - source events style: 0 - One-For-All, 1 - One-For-Every 
  */
  Generator(QProgressBar *p,QWidget *widget, QString home, QString sys, int st);

  /*!
    Destructor of class Generator
  */
  ~Generator() {};
  
  /*!
    Main function in this class. It makes source file on the basis of MyTree structure with components pointed by \e t
  */
  int generateSource(MyTree *t);

  /*!
    This function makes last things with source files generated with generateSource() function
  */
  void finishSource(QString homename, QString whatname, QString name);

  /*! 
    This function generates HTML file for applet created with BuildIT!
  */
  void genHtml(MyTree* t);

  /*! 
    Remove all comments from file
  */
  void removeComments(QString filename);

  /*!
    This function removes every three next empty lines in file \e sfilename and write output to \e dfilename
  */
  void removeSpace(QString dfilename, QString sfilename);

  /*!
    This function copys file from \e sfilename to \e dfilename. <br>
    In fact is the same like concateFiles() with \e append parameter set to FALSE
   */
  void copyFile(QString dfilename, QString sfilename);

private:

  /*!
    This function concates file named \e name2 to file named \e name1 when \e append parameter is set to TRUE, 
    when \e append=FALSE this function copy file \e name2 to file \e name2
  */
  void concateFiles(QString name1, QString name2, bool append); 

  /*!
    This function inserts file \e source2 to file \e source1 at line \e line and writes output to file \e dest
  */
  void insertFile(QString dest, QString source1, QString source2, int line); 

  /*!
    This function inserts \e buf to file \e sfilename at line \e line and writes output to \e dfilename
  */

  void insertBuffer(QString dfilename, QString sfilename, QString *buf, int line); 

  /*!
    This function adds right events function to source files
  */
  void addImport(QString filename);
  
  /*!
    \internal
  */
  void importHelp(QString filename, QString name);
  /*!
    This function adds "implements listeners section" to source file header
  */
  void addImplements(QString filename);
  
  /*!
    \internal
  */
  void implementsHelp(QString filename, QString name);

  /*!
    This function returns number of lines in file named \e filename without last empty lines
  */
  int getFileLines(QString filename);

  /*!
    This function makes part of source file from template. <br>
    The parameters are: <br>
    dfilename - name of file for output <br>
    sfilename - name of file with template <br>
    n - number of items in \e arg[] array <br>
    arg[] - strings to replace "\<markXX\>" in template file in right order (from 0 to n-1) <br>
    ap - when it's TRUE, the file named \e dfilename should be opened before calling this function <br>
  */
  void template2Source(QString dfilename, QString sfilename, int n, QString arg[], bool ap);

  /*!
    This function returns name of temporary file
  */
  QString getTmpName();

  /*!
    This function returns TRUE if \e boxgroup[] array include \e name
  */
  bool isCheckBoxGroup(QString name);

  /*!
    Add new CheckBoxGroup name to array \e boxgroup[]
  */
  void addNewCheckBoxGroup(QString name);

  /*!
    Add new Canvas \e name to \e canvastab[] array
  */
  void addNewCanvas(QString name);

  /*!
    Set \e targetname variable
  */
  void setTargetName(QString name) {targetname = name.copy();}

  /*!
    Return \e targetname variable
  */
  QString getTargetName() {return targetname.data(); }

  /*!
    Used in generate menu. Return higher menu for \e name in menubar structure pointed by \e t. <br>
    num - number of child in \e t <br>
    Function scans items in \e t from child number 0 to child number num-1
  */
  QString getHigherMenu(MyTree *t, char* name, int num);

  /*!
    This function adds right listeners buffer for component pointed by \e t. <br>
    f1name - name of temporary name <br>
    f2name - name of file where put buffer and write output <br>
    varname - name of project <br>
    line - line where put buffer into file \e f2name <br>
  */
  void addListener(MyTree *t, QString f1name, QString f2name, QString varname, int &line);

  /*!
    Used by function addListener() to set rights listeners which will be added by addImplements() function for style 0 <br>
    or for style 1 insert right pieces of code into source
  */
  void updateListener(QString name, bool tab[]);
  /*!
    \internal
  */
  void addEvent(QString filename, QString varname, int num);
  
  /*!
    \internal
  */
  void insertOneEvent(QString event, QString *tab);

  void insertAdapter(QString clname, QString adname, int from, int to);

  /*!
    This function replaces strings like "\<markXX\>" in string pointed by \e b, with strings from rep[] array. <br>
    n - number of items in \e rep[] array
  */
  QString replaceMarks(QString *b, int n, QString rep[]);
  
  //void genAddCanvas();
 
 /*!
    Generate basic applet source structure
  */
  void genApplet(MyTree *t);

  /*!
    Generate basic frame source structure
  */
  void genFrame(MyTree *t);

  /*!
    Generate basic window source structure
  */
  void genWindow(MyTree *t);

  /*!
    Generate Button  component
  */
  void genPushButton(MyTree *t);

  /*!
    Generate Checbox  component
  */
  void genCheckbox(MyTree *t);

  /*!
    Generate Scrollbar  component
  */
  void genScrollbar(MyTree *t);  

  /*!
    Generate Label  component
  */
  void genLabel(MyTree *t);

  /*!
    Generate Choice  component
  */
  void genChoice(MyTree *t);

  /*!
    Generate Canvas  component
  */
  void genCanvas(MyTree *t);

  /*!
    Generate List  component
  */
  void genList(MyTree *t);

  /*!
    Generate TexField  component
  */
  void genTextField(MyTree *t);

  /*!
    Generate TextArea  component
  */
  void genTextArea(MyTree *t);

  /*!
    Generate MenuBar  component
  */
  void genMenuBar(MyTree *t);

  /*!
    Generate Menu  component
  */
  void genMenu(MyTree *t);

  /*!
    Generate MenuItem  component
  */
  void genMenuItem(MyTree *t, QString higher);

  /*!
    Generate CheckboxMenuItem  component
  */
  void genCheckboxMenuItem(MyTree *t, QString higher);

  /*!
    Generate SubMenu  component
  */
  void genSubMenu(MyTree *t, QString higher);

  /*!
    Generate comment for source file and insert it at the begining of the source file
  */
  void genComment(QString homename, QString name);
  
  /*!
    \internal
  */
  void clearTab();

  // *************************************************************

  /*!
    \internal
  */
  int max;
  
  /*!
    \internal
  */
  int counter;

  /*!
    \internal
  */
  int tmpcounter;

  /*!
    \internal
  */
  bool first;
  /*!
    Line where to put declaration of component
  */
  int l1;
  /*! 
    Line where to put definition of component
  */
  int l2;
  /*!
    Line where to put Events definition for Style 1
    l3 = 36 when generating applet <br>
    l3 = 27 when generating frame <br>
  */
  int l3;

  /*!
    Line where to put implement
  */
  int l4;

  /*!
    Define the style of generating source
    0 - implements (all Listener functions added)
    1 - use inner classes to operate events
  */
  int style;

  /*!
    Array of strings with CheckBox groups
  */
  QString boxgroups[10];

  /*!
    Array of strings with Canvas
  */
  QString canvastab[20];

  /*!
    Array of strings with Listener names
  */
  QString listenertab[11];

  /*!
    Array of states of Listeners to put into source file
  */
  bool listener[11];

  /*!
    Array of variable's scope: private, protected, public
  */
  QString scope[3];

  /*!
    comp 0-3 <br>
    cont 4-5 <br>
    focus 6-7 <br>
    key 8-10 <br>
    mouse 11-15 <br>
    motion 16-17 <br>
    window 18-24 <br>
  */
  bool evntTab[25]; 

  /*!
    \internal
  */
  QString targetname;
  
  /*! 
    Absolute path to user directory with .build/ directory
  */
  QString homeDir;

  /*!
    Absolute path to <b> BuildIT! </b> directory with Templates and documentation files
  */
  QString sysDir;
  
  /*!
    \internal
  */
  int numgroups;

  /*!
    \internal
  */
  int numcanvas;
  
  /*!
    \internal
  */
  int menuy;

  /*!
    Used to show ProgressDialog
  */
  QProgressBar *progress;
};
